-- Analogblock1_top_level.vhd

library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
library fusion;

entity Analogblock1_top_level is
    port(
        
        SYS_RESET:  in std_logic;
    
         -- Analog Ports
         VAREF         : inout std_logic := 'Z';        
         AV0_pin       : in std_logic;                  
         

         --LCD Ports
         RS_LCD        : out std_logic; 
         DATA_LCD      : out std_logic_vector(7 downto 4); 
         EN_LCD        : out std_logic; 
         R_nW_LCD      : out std_logic; 

         --Testsignale
         DATAVALID,      
         INIT_DONE,
         ASSC_DONE      : out std_logic;
         --ASSC_WAIT      : out std_logic;        
         --ASSC_CHSAT     : out std_logic;        
         --ASSC_CHLATD    : out std_logic; 
         ASSC_RAM_WR_BUSY_B : out std_logic;
         GLA              : out std_logic;
         GLB              : out std_logic; 
         ADC_counts   : out std_logic_vector (11 downto 0)
        );
end entity Analogblock1_top_level;

architecture Analogblock1_ARCH of Analogblock1_top_level is

---------------------------components----------------------------
component ADC is
    port(
        SYS_CLK, 
        SYS_RESET           : in std_logic; 
        VAREF               : inout std_logic := 'Z'; 
        DATAVALID           : out std_logic; 
        AV0                 : in std_logic; 
        USER_ASSC_RAM_BUSY  : out std_logic; 
        USER_ASSC_ADDR      : in std_logic_vector(8 downto 0); 
        USER_ASSC_RD        : in std_logic; 
        ASSC_RAM_WR_BUSY_B  : out std_logic; 
        ASSC_RAM_DOUT       : out std_logic_vector(8 downto 0); 
        ASSC_DONE, 
        ASSC_WAIT, 
        ASSC_CHSAT, 
        ASSC_CHLATD         : out std_logic; 
        INIT_ADDR           : in std_logic_vector(8 downto 0); 
        INIT_DATA           : in std_logic_vector(8 downto 0); 
        INIT_ACM_WEN, 
        INIT_ASSC_WEN, 
        INIT_EV_WEN, 
        INIT_TR_WEN, 
        INIT_DONE           : in std_logic; 
        ADC_CHNUMBER        : out std_logic_vector(4 downto 0)
        );
end component ADC;


component NVM_ADC is

    port(
        INIT_CLK, 
        SYS_RESET, 
        INIT_POWER_UP       : in std_logic; 
        INIT_DONE           : out std_logic; 
        INIT_DATA           : out std_logic_vector(8 downto 0); 
        INIT_ADDR           : out std_logic_vector(8 downto 0); 
        INIT_ACM_WEN, 
        INIT_ASSC_WEN, 
        INIT_EV_WEN, 
        INIT_TR_WEN         : out std_logic
        );    
end component NVM_ADC;

--INBUF Used To Bring IN SYS_RESET On A Regular IO
component INBUF
   port (
        PAD            : in std_logic := 'U'; 
        Y              : out std_logic
        );
end component INBUF;

--CLKINT Used To Promote The SYS_RESET Onto Global Network
component CLKINT
    port(
        A : in std_logic := 'U'; 
        Y : out std_logic
        );
end component;

component RCO is 
    port(
        CLKOUT : out std_logic
        );
end component RCO;

component PLL_6_1 is 
    port(
        POWERDOWN,                  -- '0' power-down, '1' power-up
        CLKA : in std_logic;        -- 100 MHz from RCO
        LOCK,                       -- all requirements achieved
        GLA,                        -- 6.25 MHz
        GLB : out std_logic;        -- 1 MHz 
        OADIVRST : in std_logic) ;
end component PLL_6_1;

component VCC
    port(Y		: out    STD_ULOGIC);
end component;

component LCD_controller is
    Port ( 
        reset_n      : in std_logic;
        clock   : in std_logic; 
        ADC_counts : in std_logic_vector(11 downto 0);
        LCD_data_bus : out std_logic_vector (7 downto 4);
        LCD_enable : out std_logic;
        LCD_rs : out std_logic;
        LCD_r_nw : out std_logic;
        format_select: in std_logic_vector (2 downto 0)
        );

end component LCD_controller ;


----------------------------signal------------------------------
--Testsignale
    signal DATAVALID_net            : std_logic; --ADC-value valid
    signal ADC_CHNUMBER_net         : std_logic_vector(4 downto 0);
    signal GLA_net, GLB_net         : std_logic;

--user access to ASSC_RAM
    signal USER_ASSC_RAM_BUSY_net   : std_logic; 
    signal USER_ASSC_ADDR_bus       : std_logic_vector(8 downto 0); 
    signal USER_ASSC_RD_net         : std_logic;
    signal ASSC_RAM_DOUT_bus        : std_logic_vector(8 downto 0);

--initialization ADC
    signal INIT_ADDR_int            : std_logic_vector(8 downto 0); 
    signal INIT_DATA_int            : std_logic_vector(8 downto 0); 
    signal INIT_ACM_WEN_int         : std_logic;
    signal INIT_ASSC_WEN_int        : std_logic;
    signal INIT_EV_WEN_int          : std_logic;
    signal INIT_TR_WEN_int          : std_logic;
    signal INIT_DONE_int            : std_logic;

--reset
    signal SYS_RESET_int            : std_logic; --high-active
    signal SYS_RESET_INV            : std_logic; --low-active
    signal SYS_RESET_global         : std_logic; --low-active global net 

--CLK
    signal clk_int                  : std_logic; --global network
    signal clk_6MHz_net             : std_logic;
    signal clk_1MHz_net             : std_logic;

--sonstige
    signal LOCK                     : std_logic;
    signal ADC_counts_bus           : std_logic_vector(11 downto 0);
    signal format_net               : std_logic_vector (2 downto 0);
    signal step                     : std_logic_vector (1 downto 0);
    signal VCC_net                  : std_logic;

begin

    --testsignals
    GLA <= clk_6MHz_net;
    GLB <= clk_1MHz_net;
    ADC_counts <= ADC_counts_bus;
    INIT_DONE <=  INIT_DONE_int;
    DATAVALID <= DATAVALID_net;
    
    --global reset
    INBUF_inst: INBUF
    port map(
        PAD                 => SYS_RESET,   --high-active
        Y                   => SYS_RESET_int
        );
    
    SYS_RESET_INV <= not SYS_RESET_int;


    --CLKINT Used To Promote The SYS_RESET Onto Global Network
    SYS_RESET_CLKINT_inst: CLKINT
    port map(
        A                   => SYS_RESET_INV, 
        Y                   => SYS_RESET_global
        );


    RCO_inst: RCO 
    port map(
        CLKOUT              =>  clk_int     --clkint integrated (global network)
        );


    PLL_6_1_inst: PLL_6_1 
    port map(               
        POWERDOWN           => '1',         
        CLKA                => clk_int,  
        LOCK                => LOCK, 
        GLA                 => clk_6MHz_net, 
        GLB                 => clk_1MHz_net,  
        OADIVRST            => SYS_RESET_INV
        );



    ADC_inst: ADC
    port map(
        SYS_CLK             => clk_6MHz_net,
        SYS_RESET           => SYS_RESET_global, 
        VAREF               => VAREF,
        DATAVALID           => DATAVALID_net,
        AV0                 => AV0_pin,
        USER_ASSC_RAM_BUSY  => USER_ASSC_RAM_BUSY_net,
        USER_ASSC_ADDR      => USER_ASSC_ADDR_bus,
        USER_ASSC_RD        => USER_ASSC_RD_net, 
        ASSC_RAM_WR_BUSY_B  => ASSC_RAM_WR_BUSY_B,
        ASSC_RAM_DOUT       => ASSC_RAM_DOUT_bus,
        ASSC_DONE           => ASSC_DONE,
        --ASSC_WAIT           => ASSC_WAIT,
        --ASSC_CHSAT          => ASSC_CHSAT,
        --ASSC_CHLATD         => ASSC_CHLATD, 
        INIT_ADDR           => INIT_ADDR_int, 
        INIT_DATA           => INIT_DATA_int,
        INIT_ACM_WEN        => INIT_ACM_WEN_int,
        INIT_ASSC_WEN       => INIT_ASSC_WEN_int,
        INIT_EV_WEN         => INIT_EV_WEN_int,
        INIT_TR_WEN         => INIT_TR_WEN_int,
        INIT_DONE           => INIT_DONE_int, 
        ADC_CHNUMBER        => ADC_CHNUMBER_net
        );


    NVM_ADC_inst: NVM_ADC
    port map(
        INIT_CLK            => clk_6MHz_net,
        SYS_RESET           => SYS_RESET_global,
        INIT_POWER_UP       => VCC_net,
        INIT_DONE           => INIT_DONE_int, 
        INIT_DATA           => INIT_DATA_int, 
        INIT_ADDR           => INIT_ADDR_int,
        INIT_ACM_WEN        => INIT_ACM_WEN_int,
        INIT_ASSC_WEN       => INIT_ASSC_WEN_int,
        INIT_EV_WEN         => INIT_EV_WEN_int,
        INIT_TR_WEN         => INIT_TR_WEN_int
        );  
          
    VCC_inst: VCC
        port map(
            Y		        => VCC_net);


    LCD_controller_inst: LCD_controller
    port map( 
        reset_n             => SYS_RESET_global,
        clock               => clk_1MHz_net,
        ADC_counts          => ADC_counts_bus,
        LCD_data_bus        => DATA_LCD,
        LCD_enable          => EN_LCD,
        LCD_rs              => RS_LCD,
        LCD_r_nw            => R_nW_LCD,
        format_select       => format_net
        );

--Fetch data from ASSC RAM
    fetch_data: process (clk_1MHz_net, SYS_RESET_global, ADC_counts_bus, USER_ASSC_ADDR_bus, 
                           USER_ASSC_RD_net, USER_ASSC_RD_net, USER_ASSC_RAM_BUSY_net, step )

    begin
        if SYS_RESET_global = '0' then
           ADC_counts_bus <= (others => '0');
           step <= "00";
           USER_ASSC_RD_net <= '0';
           USER_ASSC_ADDR_bus <= (others => '0');
        elsif (clk_1MHz_net'event and clk_1MHz_net = '1') then
            case step is
                when "00" => 
                    USER_ASSC_ADDR_bus <= "000000011"; --ASSC_address 3+4 
                    format_net<= "001";                
                    step <= "01";
                when "01" =>
                    -- activate RD
                    USER_ASSC_RD_net <= '1';
                    step <= "10";
                when "10" =>
                    -- check BUSY and latch first word
                    if USER_ASSC_RAM_BUSY_net = '1' then
                        step <= "10";
                    else 
                        ADC_counts_bus(8 downto 0) <= ASSC_RAM_DOUT_bus;
                        USER_ASSC_ADDR_bus <= USER_ASSC_ADDR_bus + 1;
                        step <= "11";
                    end if;
                when "11" =>
                    -- check BUSY and latch second word
                    if USER_ASSC_RAM_BUSY_net = '1' then
                        step <= "11";
                    else
                        ADC_counts_bus(11 downto 9) <= ASSC_RAM_DOUT_bus(2 downto 0);
                        USER_ASSC_RD_net <= '0';
                        step <= "00";
                    end if;
                when others =>
                    step <= "00";
            end case;
        end if;
    end process;


end architecture Analogblock1_ARCH;
