library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity LCD_controller is
    Port ( 
        reset_n      : in std_logic;
        clock   : in std_logic; -- 1MHz
        ADC_counts : in std_logic_vector(11 downto 0);
        LCD_data_bus : out std_logic_vector (7 downto 4);
        LCD_enable : out std_logic;
        LCD_rs : out std_logic;
        LCD_r_nw : out std_logic;
        format_select: in std_logic_vector (2 downto 0)
        );

end LCD_controller ;

architecture behave of LCD_controller is


component LCD_interface is
    Port ( 
        clock        : in std_logic;
        reset_n      : in std_logic;
        input_data   : in std_logic_vector (63 downto 0);
        LCD_data_bus : out std_logic_vector (7 downto 4);
        LCD_enable : out std_logic;
        LCD_rs : out std_logic;
        LCD_r_nw : out std_logic 
        );
end component;

component formatter is
    Port ( 
        reset_n      : in std_logic;
        clock : in std_logic;
        format_select: in std_logic_vector (2 downto 0);
        counts_in : in std_logic_vector (11 downto 0);
        data_out : out std_logic_vector (63 downto 0)
        );
end component ;

signal formatted_data : std_logic_vector (63 downto 0);
signal LCD_clock : std_logic;
signal prescale : std_logic_vector (12 downto 0); -- makes LCD clock ~120Hz

begin

make_LCD_clock: process (reset_n, clock)
    begin
    if reset_n = '0' then
        prescale <= (others => '0');
    elsif clock'event and clock = '1' then
        prescale <= prescale + 1;
    end if;
    end process;
LCD_clock <= prescale(prescale'left);

formatter_inst: formatter 
    port map ( 
        reset_n => reset_n,
        clock => clock,
        format_select => format_select,
        counts_in => ADC_counts,
        data_out => formatted_data
        );

LCD_inst: LCD_interface
    port map (
        clock => LCD_clock,
        reset_n => reset_n,
        input_data => formatted_data,
        LCD_data_bus => LCD_data_bus,
        LCD_enable => LCD_enable,
        LCD_rs => LCD_rs,
        LCD_r_nw => LCD_r_nw
        );

end behave;


