library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity LCD_interface is
    Port ( 
        clock        : in std_logic;
        reset_n      : in std_logic;
        input_data   : in std_logic_vector (63 downto 0);
        LCD_data_bus : out std_logic_vector (7 downto 4);
        LCD_enable : out std_logic;
        LCD_rs : out std_logic;
        LCD_r_nw : out std_logic 
        );

end LCD_interface ;


architecture behave of LCD_interface is

type display_state is (warmup, init1, init2, clear1, clear2, setaddr1, setaddr2, setmode1, setmode2, write1, write2);

signal state : display_state;

signal count : std_logic_vector(2 downto 0);


begin
	LCD_r_nw <= '0';    -- always read
    LCD_enable <= clock;

    drive_LCD: process (clock, reset_n)
	begin
		if reset_n = '0' then
			count <= (others => '0');
            state <= warmup;
			LCD_data_bus <= "0000";	
			LCD_rs <= '0';

		elsif (clock'event and clock = '1') then
			case state is
				when warmup =>
					LCD_rs <= '0';
					LCD_data_bus <= "0000";         -- 0x00 = do nothing. Needs 15ms warmup.
					if count = "011" then           -- for 25Hz clock, 3 cycles = 120ms
						count <= (others => '0');
    					LCD_rs <= '0';
                        LCD_data_bus <= "0010";     -- 0x20 = 4 bit interface, 1 line, 5x7 font
						state <= init1;
					else
						count <= count + 1;
						state <= warmup;
					end if;
					
				when init1 =>
                    LCD_rs <= '0';
                    LCD_data_bus <= "0000";
                    state <= init2;
                when init2 =>
					LCD_rs <= '0';
                    LCD_data_bus <= "0000";         -- 0x01 = clear display
                    state <= clear1;
				
                when clear1 =>
					LCD_rs <= '0';
                    LCD_data_bus <= "0001";
                    state <= clear2;
                when clear2 =>
                    LCD_rs <= '0';
                    LCD_data_bus <= "0000";     -- 0x0C = display on, cursor off
                    state <= setmode1;
                    
                when setmode1 =>
					LCD_rs <= '0';
                    LCD_data_bus <= "1100";
                    state <= setmode2;
				when setmode2 =>
                    LCD_rs <= '0';
                    LCD_data_bus <= "1000";      -- 0x80 to 0x87 = 8 LCD character addresses
					state <= setaddr1;
                                    											
				when setaddr1 =>
                    LCD_rs <= '0';
                    LCD_data_bus <= '0' & count;
                    state <= setaddr2;
                when setaddr2 =>
                    LCD_rs <= '1';               -- write data to DDRAM
                    LCD_data_bus <= input_data((63-8*conv_integer(count)) downto (60-8*conv_integer(count)));
                    state <= write1;
            
                when write1 =>
                    LCD_rs <= '1';
                    LCD_data_bus <= input_data((59-8*conv_integer(count)) downto (56-8*conv_integer(count)));
                    state <= write2;
                when write2 =>
                    count <= count + 1;
                    LCD_rs <= '0';
                    LCD_data_bus <= "1000";     -- upper nibble of address (as in setmode2)
                    state <= setaddr1;
			end case;
		end if;
	end process;

end behave;
